==> bucketlib-fabric-1.21.3-4.2.0.7-sources.jar: de/cech12/bucketlib/api/crafting/EntityIngredient.java <==
package de.cech12.bucketlib.api.crafting;

import com.mojang.serialization.MapCodec;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import de.cech12.bucketlib.BucketLibMod;
import de.cech12.bucketlib.api.BucketLib;
import de.cech12.bucketlib.util.BucketLibUtil;
import de.cech12.bucketlib.util.RegistryUtil;
import net.fabricmc.fabric.api.recipe.v1.ingredient.CustomIngredient;
import net.fabricmc.fabric.api.recipe.v1.ingredient.CustomIngredientSerializer;
import net.minecraft.class_1299;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_2960;
import net.minecraft.class_6862;
import net.minecraft.class_6880;
import net.minecraft.class_7923;
import net.minecraft.class_7924;
import net.minecraft.class_9129;
import net.minecraft.class_9139;
import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

public class EntityIngredient implements CustomIngredient {

    protected final class_1299<?> entityType;
    protected final class_6862<class_1299<?>> tag;
    private List<class_6880<class_1792>> matchingStacks;

    private EntityIngredient(class_1299<?> entityType, class_6862<class_1299<?>> tag) {
        this.entityType = entityType;
        this.tag = tag;
    }

    public EntityIngredient(Optional<class_2960> blockOptional, Optional<class_6862<class_1299<?>>> tagOptional) {
        this(blockOptional.map(class_7923.field_41177::method_10223).filter(Optional::isPresent).map(reference -> reference.get().comp_349()).orElse(null), tagOptional.orElse(null));
    }

    public EntityIngredient(class_1299<?> entityType) {
        this(entityType, null);
    }

    public EntityIngredient(class_6862<class_1299<?>> tag) {
        this(null, tag);
    }

    @Override
    public boolean test(class_1799 itemStack) {
        if (itemStack == null || itemStack.method_7960()) {
            return false;
        }
        List<RegistryUtil.BucketEntity> bucketEntities;
        if (this.entityType != null) {
            RegistryUtil.BucketEntity bucketEntity = RegistryUtil.getBucketEntity(this.entityType);
            if (bucketEntity == null) {
                return false;
            }
            bucketEntities = List.of(bucketEntity);
        } else {
            bucketEntities = RegistryUtil.getBucketEntities().stream().filter(bucketBlock -> bucketBlock.entityType().method_20210(this.tag)).toList();
        }
        for (RegistryUtil.BucketEntity bucketEntity : bucketEntities) {
            if (itemStack.method_7909() == bucketEntity.bucketItem()) {
                return true;
            }
            return BucketLibUtil.getEntityType(itemStack) == bucketEntity.entityType();
        }
        return false;
    }

    @Override
    public List<class_6880<class_1792>> getMatchingItems() {
        if (this.matchingStacks == null) {
            this.matchingStacks = new ArrayList<>();
            List<class_1299<?>> entityTypes = new ArrayList<>();
            if (this.tag != null) {
                class_7923.field_41177.method_40286(this.tag).forEach(fluid -> entityTypes.add(fluid.comp_349()));
            } else if (this.entityType != null) {
                entityTypes.add(this.entityType);
            }
            List<RegistryUtil.BucketEntity> bucketEntities = RegistryUtil.getBucketEntities().stream().filter(bucketEntity -> entityTypes.contains(bucketEntity.entityType())).toList();
            //vanilla buckets
            for (RegistryUtil.BucketEntity bucketEntity : bucketEntities) {
                this.matchingStacks.add(class_6880.method_40223(bucketEntity.bucketItem()));
            }
            //bucket lib buckets
            for (RegistryUtil.BucketEntity bucketEntity : bucketEntities) {
                BucketLibMod.getRegisteredBuckets().forEach(bucket -> {
                    if (bucket.canHoldFluid(bucketEntity.fluid()) && bucket.canHoldEntity(bucketEntity.entityType())) {
                        /*
                        ItemStack filledBucket = new ItemStack(bucket);
                        if (bucketEntity.fluid() != Fluids.EMPTY) {
                            filledBucket = BucketLibUtil.addFluid(filledBucket, bucketEntity.fluid());
                        }
                        filledBucket = BucketLibUtil.addEntityType(filledBucket, bucketEntity.entityType());
                        this.matchingStacks.add(filledBucket);
                         */
                        this.matchingStacks.add(class_6880.method_40223(bucket));
                    }
                });
            }
        }
        return this.matchingStacks;
    }

    @Override
    public boolean requiresTesting() {
        return true;
    }

    @Override
    public CustomIngredientSerializer<?> getSerializer() {
        return Serializer.INSTANCE;
    }

    public static final class Serializer implements CustomIngredientSerializer<EntityIngredient> {

        public static final Serializer INSTANCE = new Serializer();
        public static final class_2960 NAME = BucketLib.id("entity");

        public static final MapCodec<EntityIngredient> CODEC = RecordCodecBuilder.mapCodec(builder ->
                builder.group(
                        class_2960.field_25139.optionalFieldOf("entity").forGetter(i -> Optional.of(class_7923.field_41177.method_10221(i.entityType))),
                        class_6862.method_40090(class_7923.field_41177.method_46765()).optionalFieldOf("tag").forGetter(i -> Optional.ofNullable(i.tag))
                ).apply(builder, EntityIngredient::new)
        );

        private static final class_9139<class_9129, EntityIngredient> PACKET_CODEC = class_9139.method_56437(
                EntityIngredient.Serializer::write,
                EntityIngredient.Serializer::read);

        private Serializer() {}

        @Override
        public class_2960 getIdentifier() {
            return NAME;
        }

        @Override
        public MapCodec<EntityIngredient> getCodec() {
            return CODEC;
        }

        @Override
        public class_9139<class_9129, EntityIngredient> getPacketCodec() {
            return PACKET_CODEC;
        }

        @Nonnull
        private static EntityIngredient read(class_9129 buffer) {
            String entity = buffer.method_19772();
            String tagId = buffer.method_19772();
            if (!tagId.isEmpty()) {
                class_6862<class_1299<?>> tag = class_6862.method_40092(class_7924.field_41266, class_2960.method_60654(tagId));
                return new EntityIngredient(tag);
            }
            if (entity.isEmpty()) {
                throw new IllegalArgumentException("Cannot create a entity ingredient with no entity or tag.");
            }
            Optional<class_6880.class_6883<class_1299<?>>> entityTypeOptional = class_7923.field_41177.method_10223(class_2960.method_60654(entity));
            if (entityTypeOptional.isEmpty()) {
                throw new IllegalArgumentException("Entity type resource location \"" + entity + " \" could not be found.");
            }
            return new EntityIngredient(entityTypeOptional.get().comp_349());
        }

        private static void write(@Nonnull class_9129 buffer, @Nonnull EntityIngredient ingredient) {
            buffer.method_10814(ingredient.entityType != null ? Objects.requireNonNull(class_7923.field_41177.method_10221(ingredient.entityType)).toString() : "");
            buffer.method_10814(ingredient.tag != null ? ingredient.tag.comp_327().toString() : "");
        }
    }

}

==> bucketlib-fabric-1.21-4.1.6.0-sources.jar: de/cech12/bucketlib/api/crafting/EntityIngredient.java <==
package de.cech12.bucketlib.api.crafting;

import com.mojang.serialization.MapCodec;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import de.cech12.bucketlib.BucketLibMod;
import de.cech12.bucketlib.api.BucketLib;
import de.cech12.bucketlib.util.BucketLibUtil;
import de.cech12.bucketlib.util.RegistryUtil;
import net.fabricmc.fabric.api.recipe.v1.ingredient.CustomIngredient;
import net.fabricmc.fabric.api.recipe.v1.ingredient.CustomIngredientSerializer;
import net.minecraft.class_1299;
import net.minecraft.class_1799;
import net.minecraft.class_2960;
import net.minecraft.class_3612;
import net.minecraft.class_6862;
import net.minecraft.class_6885;
import net.minecraft.class_7923;
import net.minecraft.class_7924;
import net.minecraft.class_9129;
import net.minecraft.class_9139;
import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

public class EntityIngredient implements CustomIngredient {

    protected final class_1299<?> entityType;
    protected final class_6862<class_1299<?>> tag;
    private List<class_1799> matchingStacks;

    private EntityIngredient(class_1299<?> entityType, class_6862<class_1299<?>> tag) {
        this.entityType = entityType;
        this.tag = tag;
    }

    public EntityIngredient(Optional<class_2960> blockOptional, Optional<class_6862<class_1299<?>>> tagOptional) {
        this(blockOptional.map(class_7923.field_41177::method_10223).orElse(null), tagOptional.orElse(null));
    }

    public EntityIngredient(class_1299<?> entityType) {
        this(entityType, null);
    }

    public EntityIngredient(class_6862<class_1299<?>> tag) {
        this(null, tag);
    }

    @Override
    public boolean test(class_1799 itemStack) {
        if (itemStack == null || itemStack.method_7960()) {
            return false;
        }
        List<RegistryUtil.BucketEntity> bucketEntities;
        if (this.entityType != null) {
            RegistryUtil.BucketEntity bucketEntity = RegistryUtil.getBucketEntity(this.entityType);
            if (bucketEntity == null) {
                return false;
            }
            bucketEntities = List.of(bucketEntity);
        } else {
            bucketEntities = RegistryUtil.getBucketEntities().stream().filter(bucketBlock -> bucketBlock.entityType().method_20210(this.tag)).toList();
        }
        for (RegistryUtil.BucketEntity bucketEntity : bucketEntities) {
            if (itemStack.method_7909() == bucketEntity.bucketItem()) {
                return true;
            }
            return BucketLibUtil.getEntityType(itemStack) == bucketEntity.entityType();
        }
        return false;
    }

    @Override
    public List<class_1799> getMatchingStacks() {
        if (this.matchingStacks == null) {
            this.matchingStacks = new ArrayList<>();
            List<class_1299<?>> entityTypes = new ArrayList<>();
            Optional<class_6885.class_6888<class_1299<?>>> entityTag = Optional.empty();
            if (this.tag != null) {
                entityTag = class_7923.field_41177.method_40266(this.tag);
            }
            if (entityTag.isPresent()) {
                entityTag.get().forEach(fluid -> entityTypes.add(fluid.comp_349()));
            } else if (this.entityType != null) {
                entityTypes.add(this.entityType);
            }
            List<RegistryUtil.BucketEntity> bucketEntities = RegistryUtil.getBucketEntities().stream().filter(bucketEntity -> entityTypes.contains(bucketEntity.entityType())).toList();
            //vanilla buckets
            for (RegistryUtil.BucketEntity bucketEntity : bucketEntities) {
                this.matchingStacks.add(new class_1799(bucketEntity.bucketItem()));
            }
            //bucket lib buckets
            for (RegistryUtil.BucketEntity bucketEntity : bucketEntities) {
                BucketLibMod.getRegisteredBuckets().forEach(bucket -> {
                    if (bucket.canHoldFluid(bucketEntity.fluid()) && bucket.canHoldEntity(bucketEntity.entityType())) {
                        class_1799 filledBucket = new class_1799(bucket);
                        if (bucketEntity.fluid() != class_3612.field_15906) {
                            filledBucket = BucketLibUtil.addFluid(filledBucket, bucketEntity.fluid());
                        }
                        filledBucket = BucketLibUtil.addEntityType(filledBucket, bucketEntity.entityType());
                        this.matchingStacks.add(filledBucket);
                    }
                });
            }
        }
        return this.matchingStacks;
    }

    @Override
    public boolean requiresTesting() {
        return true;
    }

    @Override
    public CustomIngredientSerializer<?> getSerializer() {
        return Serializer.INSTANCE;
    }

    public static final class Serializer implements CustomIngredientSerializer<EntityIngredient> {

        public static final Serializer INSTANCE = new Serializer();
        public static final class_2960 NAME = BucketLib.id("entity");

        public static final MapCodec<EntityIngredient> CODEC = RecordCodecBuilder.mapCodec(builder ->
                builder.group(
                        class_2960.field_25139.optionalFieldOf("entity").forGetter(i -> Optional.of(class_7923.field_41177.method_10221(i.entityType))),
                        class_6862.method_40090(class_7923.field_41177.method_30517()).optionalFieldOf("tag").forGetter(i -> Optional.ofNullable(i.tag))
                ).apply(builder, EntityIngredient::new)
        );

        private static final class_9139<class_9129, EntityIngredient> PACKET_CODEC = class_9139.method_56437(
                EntityIngredient.Serializer::write,
                EntityIngredient.Serializer::read);

        private Serializer() {}

        @Override
        public class_2960 getIdentifier() {
            return NAME;
        }

        @Override
        public MapCodec<EntityIngredient> getCodec(boolean allowEmpty) {
            return CODEC;
        }

        @Override
        public class_9139<class_9129, EntityIngredient> getPacketCodec() {
            return PACKET_CODEC;
        }

        @Nonnull
        private static EntityIngredient read(class_9129 buffer) {
            String entity = buffer.method_19772();
            String tagId = buffer.method_19772();
            if (!tagId.isEmpty()) {
                class_6862<class_1299<?>> tag = class_6862.method_40092(class_7924.field_41266, class_2960.method_60654(tagId));
                return new EntityIngredient(tag);
            }
            if (entity.isEmpty()) {
                throw new IllegalArgumentException("Cannot create a entity ingredient with no entity or tag.");
            }
            return new EntityIngredient(class_7923.field_41177.method_10223(class_2960.method_60654(entity)));
        }

        private static void write(@Nonnull class_9129 buffer, @Nonnull EntityIngredient ingredient) {
            buffer.method_10814(ingredient.entityType != null ? Objects.requireNonNull(class_7923.field_41177.method_10221(ingredient.entityType)).toString() : "");
            buffer.method_10814(ingredient.tag != null ? ingredient.tag.comp_327().toString() : "");
        }
    }

}
